/*
 * Copyright (c) 1997-2000 SuSE GmbH Nuernberg
 * Copyright (c) 2022 SUSE Software Solutions Germany GmbH
 * Version 0.7
 * Author:  Werner Fink,  <werner@suse.de>
 */
static const char unused[] __attribute__ ((unused)) = "\n"
"\000***********************************************\n"
"\000* Copyright (c) 1997-2022 SuSE GmbH Nuernberg *\n"
"\000* Copyright (c) 2022 SUSE Software Solutions Germany GmbH *\n"
"\000* Author:  Werner Fink,  <werner@suse.de>     *\n"
"\000***********************************************\n";

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <errno.h>
#include <signal.h>
#include <string.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#ifndef MAIN
#  include "dm.h"
#  include "greet.h"
#endif
#include <X11/xpm.h>

extern int get_pick(const char *input, const int ilen,
    char **output, u_int *olen);
extern void calcpixpos(const Display *dpy, const char *winname,
    u_int pixw, u_int pixh, u_int *pixx, u_int *pixy);
#ifdef MAIN
static void setbackgrd (Display *dpy, char *winname, char *backbuf);
#endif

#include "SuSE_Linux.h"

#ifdef MAIN
int main ()
{
    Display *dpy;
    FILE * release = fopen("/etc/os-release", "r");
    char *display_name = getenv("DISPLAY");
    u_int out_len;
    char *output;

    dpy = XOpenDisplay(display_name);
    if (!dpy) {
	fprintf(stderr, "Cannot open Display!\n");
	fprintf(stderr, "Please check or set your DISPLAY variable\n");
	exit(-1);
    }

    if (!release)
	get_pick(SLES_data, SLES_len, &output, &out_len);
    else {
    	char buf[128];
	char *ptr;
	ptr = fgets(buf, sizeof(buf)-1, release);
	fclose(release);
	if ((ptr = strchr(buf, '"')))
	    ptr++;
	else
	    ptr = &buf[0];
	if (strncasecmp(ptr, "opensuse", 8) == 0)
	    get_pick(openSuSE_data, openSuSE_len, &output, &out_len);
	else
	    get_pick(SLES_data, SLES_len, &output, &out_len);
    }
    setbackgrd(dpy, _PROGRAM, output);
    free(output);

    XCloseDisplay(dpy);
    return 0;
}
#endif

#ifdef MAIN
static void setbackgrd (Display *dpy, char *winname, char *backbuf)
#else
void        setbackgrd (Display *dpy, char *winname, char *backbuf)
#endif
{
    int format, screen, val, pseudocolor = 0, color_ch = 0, clear_past = 0;
    u_long length, after;
    u_char *data;
    u_int x_off, y_off;
    Window root;
    XWindowAttributes rootattr;
    Colormap cmap;
    Pixmap backxpm, backshp, pix = (Pixmap)0;
    XpmAttributes backattr;
    XGCValues gcv;
    GC gc;
    XColor c;
    Atom prop, type;

    if (!dpy)
	return;
    screen = DefaultScreen(dpy);
    gc     = DefaultGC(dpy, screen);
    cmap   = DefaultColormap(dpy, screen);
    root   = RootWindow(dpy, screen);
    (void)XGetWindowAttributes(dpy, root, &rootattr);
    pseudocolor = (DefaultVisual(dpy, screen)->class & 1);

    /* Set up base screen attributes */
    backattr.valuemask = (XpmSize|XpmReturnPixels|XpmColormap|XpmReturnColorTable);
    backattr.colormap  = rootattr.colormap;
    backattr.depth     = rootattr.depth;
    backattr.ncolors   = 0;

    val = XpmCreatePixmapFromBuffer(dpy, root, backbuf,
				    &backxpm, &backshp, &backattr);
    switch (val) {
	case XpmColorFailed:
	    fprintf(stderr, "Couldn't allocate required colors\n");
	    goto outhere;
	case XpmColorError:
	    fprintf(stderr, "Invalid Color specified in BackGround Xpm\n");
	    goto outhere;
	case XpmNoMemory:
	    fprintf(stderr, "Out of Memory\n");
	    goto outhere;
	default:
	    break;
    }

    if (!backattr.ncolors) {
	fprintf(stderr, "No colors found in BackGround Xpm\n");
	goto outhere;
    }

    /* where we should place the pixmap? */
    calcpixpos(dpy, winname, backattr.width, backattr.height, &x_off, &y_off);
    if (x_off == 0 || y_off == 0) {
	goto outhere;
    }

    /* background color */
    if (!XParseColor(dpy, cmap, backattr.colorTable[0].c_color, &c)) {
	goto outhere;
    }
    if (!XAllocColor(dpy, cmap ,&c)) {
	goto outhere;
    }
    if ((c.pixel != BlackPixel(dpy, screen)) &&
	(c.pixel != WhitePixel(dpy, screen)) && pseudocolor )
	color_ch = 1;

    /* let's us create a new root pixmap */
    pix = XCreatePixmap(dpy, root,
			rootattr.width, rootattr.height, rootattr.depth);
    XSync(dpy, False);

    /* background pixmap for the new root pixmap */
    if (backshp) {
	gcv.clip_mask=backshp;     /* set clip mask */
	gcv.clip_x_origin=x_off;   /* and clip origin */
	gcv.clip_y_origin=y_off;
	XChangeGC(dpy, gc, (GCClipMask|GCClipXOrigin|GCClipYOrigin|GCBackground), &gcv);
    }

    /* new background for root pixmap:
     * set and fill up
     */
    XSetForeground(dpy, gc, c.pixel); /* Background is the foreground of the background */
    XFillRectangle(dpy, pix, gc, 0, 0,
		   DisplayWidth(dpy, screen), DisplayHeight(dpy, screen));

    /* our pixmap should go to the new root pixmap */
    XCopyArea(dpy, backxpm, pix, gc, 0, 0,
	      backattr.width, backattr.height, x_off, y_off);

    /* now it's time to set the hole thing */
    XSetWindowBackgroundPixmap(dpy, root, pix);
    if (!color_ch) {
	XFreePixmap(dpy, pix);
	pix = (Pixmap)0;
    }

    /* just do it */
    XClearWindow(dpy, root);
    clear_past = 1;

    /* check if we have to fix up the state */
    if (!pseudocolor)
	clear_past = 0;
    if (!clear_past && !color_ch)
	goto outhere;

    /* here we are -- fixing up state for PseudoColor
     * this will save colors in colormap
     */
    prop = XInternAtom(dpy, "_XSETROOT_ID", False);
    if (clear_past) {
	XGetWindowProperty(dpy, root, prop, 0L, 1L, True, AnyPropertyType,
			   &type, &format, &length, &after, &data);

	if ((type == XA_PIXMAP) &&
	    (format == 32) && (length == 1) && (after == 0)) {
	    XKillClient(dpy, *((Pixmap *)data));
	} else if (type != 0)
	    fprintf(stderr, "Warning: _XSETROOT_ID property is garbage\n");
    }
    if (color_ch) {
	if (!pix) pix = XCreatePixmap(dpy, root, 1, 1, 1);
	if (!pix) goto outhere;
	XChangeProperty(dpy, root, prop, XA_PIXMAP, 32,
			PropModeReplace, (u_char *) &pix, 1);
	XSetCloseDownMode(dpy, RetainPermanent);
    }

outhere:
    if (backshp)   XFreePixmap(dpy, backshp);
    if (backxpm)   XFreePixmap(dpy, backxpm);
    XpmFreeAttributes(&backattr);

    XFlush(dpy);
}

/*
 * end
 */
