/*
 * login-pos.c
 *
 * Copyright (c) 1997-2000 SuSE GmbH Nuernberg
 * Version 0.4
 * Author:  Werner Fink,  <werner@suse.de>
 *
 * Get the position of an X client to place a
 * pixmap a the top ... or bottom of the client.
 *
 */
#include <stdio.h>
#include <stddef.h>
#include <unistd.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/xpm.h>

static Window windowbyname(const Display *dpy,
    const Window top, const char *who)
{
    Window *child, dummy, w=0;
    u_int nchild;
    int i;
    char *wname;

    if (XFetchName((Display *)dpy, top, &wname) && !strcmp(wname, who))
	return(top);

    if (!XQueryTree((Display *)dpy, top, &dummy, &dummy, &child, &nchild))
	return(0);

    for (i=0; i<nchild; i++) {
	w = windowbyname(dpy, child[i], who);
	if (w)
	    break;
    }
    if (child) XFree ((char *)child);
    return(w);
}


static void getwindowgeo(const Display *dpy, const char *name, int *px, int *py,
		  u_int *pw, u_int *ph, u_int *brdw, u_int *depth)
{
    Window root, winofname, dummy;
    XWindowAttributes attr;
    int screen;

    screen = DefaultScreen(dpy);
    root = RootWindow(dpy, screen);
    
    if ( (winofname = windowbyname(dpy, root, name)) == 0 ) {
	(*px) = (*py) = (*pw) = (*ph) = (*brdw) = (*depth) = 0;
	return;
    }

    XGetWindowAttributes ((Display *)dpy, winofname, &attr);
    XTranslateCoordinates((Display *)dpy, winofname, attr.root, 
			  -attr.border_width, -attr.border_width,
			  px, py, &dummy);
    *pw    = attr.width;
    *ph    = attr.height;
    *brdw  = attr.border_width;
    *depth = attr.depth;

    return;
}

void calcpixpos(const Display *dpy, const char *winname,
    u_int pixw, u_int pixh, u_int *pixx, u_int *pixy)
{
    int screen = DefaultScreen(dpy);
    u_int wd = DisplayWidth ((Display*)dpy, screen);
    u_int ht = DisplayHeight((Display*)dpy, screen);
    int px, py, tmpx, tmpy;
    u_int pw, ph, brdw, depth;

    *pixx = 0;
    *pixy = 0;
    getwindowgeo(dpy, winname, &px, &py, &pw, &ph, &brdw, &depth);

    pw += 2*brdw;
    ph += 2*brdw;

    if (pixw > wd || pixh > ht || pw > wd || ph > ht) {
	/* our pixmap is useless */
	return;
    }

    if ((int)py >= (int)(pixh + 2)) {
	/* we have enough space above winname */
	tmpy = (int)py - (int)(pixh + 1);
    } else if ((int)ht - (int)(py + ph) >= (int)(pixh + 2)) {
	/* we have enough space under winname */ 
	tmpy = (int)(py + ph + 1);
    } else
	return;

    /* first we center our pixmap */
    tmpx = px + (int)(0.5 * (double)((int)pw - (int)pixw));
    if ((int)(tmpx + pixw) >= (int)wd) {
	/* oops, three is no space on the left  side */
	tmpx = (int)wd - (int)(pixw + 1);
    } else if ((tmpx) <= 0) {
	/* oops, three is no space on the right side */
	tmpx = 1;
    }

    if (tmpx < 0 && tmpy < 0)
	return;
    *pixx = (u_int)tmpx;
    *pixy = (u_int)tmpy;

#ifdef MAIN
    fprintf (stderr, "%d %d %u %u %u %u\n", px, py, pw, ph, pixw, pixh);
    fprintf (stderr, "%u %u\n", *pixx, *pixy);
#endif

    return;
}

#ifdef MAIN
void main()
{

    char *display_name = NULL;
    Display *dpy;
    int px, py;

    dpy = XOpenDisplay(display_name);
    if (!dpy)
        return;

    calcpixpos(dpy, "xconsole", 474, 106, &px, &py);

}
#endif
