/*
 * unzip.c
 *
 * Copyright (c) 1997-2000 SuSE GmbH Nuernberg
 * Version 0.4
 * Author: Werner Fink, <werner@suse.de>
 *
 * Tools to unzip gziped data buffer encapsulated into the
 * program its self.
 *
 */
#include <stdio.h>
#include <malloc.h>
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <limits.h>
#include <sys/types.h>

#define   USEFDOPEN	1
#ifdef    USEZLIB
#  undef  USEZLIB
#  undef  USEFDOPEN
#  define USEZLIB	1
#endif

#ifdef USEZLIB
/*
 * some defs and decs from /usr/include/zlib.h
 */
typedef void* voidp;
typedef voidp gzFile;
extern gzFile gzdopen __P((int fd, char *mode));
extern int    gzread  __P((gzFile file, voidp buf, unsigned len));
extern int    gzclose __P((gzFile file));
#endif

#ifdef MAIN
#  include "SuSE_Linux.h"
#endif

int get_pick(const char *input, const int ilen, char **output, u_int *olen)
{
    u_char  *inbuf = (u_char*)input;
    u_int   size = 0;
    u_long  orig_len;
#ifdef USEFDOPEN
    int     pid, fdto[2], fdfrom[2], err;
#else
    int     pid, fdto[2], fdfrom[2], mark, err;
#endif

    *output = (char*)0;
    *olen   = 0;

    /* get the original length at the end of the compressed data */
    orig_len  = (u_long)(inbuf[ilen-4]);
    orig_len |= (u_long)(inbuf[ilen-3]) << 8;
    orig_len |= (u_long)(inbuf[ilen-2]) << 16;
    orig_len |= (u_long)(inbuf[ilen-1]) << 24;

    if (pipe(fdto) < 0 || pipe(fdfrom) < 0) {
	perror("pipe");
	return -1;
    }

    switch ((pid = fork ())) {
    case -1:
	perror("fork");
	return -1;
    case  0: { /* it's own group */
#if USEZLIB
        gzFile bytestream;
	char buffer[orig_len + 3];
#endif
	fsync(0);
	close(0);		/* close stdin */
	err = dup(fdto[0]);	/* copy fdto[0] on fileno(stdin) */
	if (err < 0) {
	    perror("dup");
	    exit(err);
	}
        close(fdto[0]);
	fsync(0);

	fsync(1);
        close(1);		/* close stdout */
	err = dup(fdfrom[1]);	/* copy fdfrom[1] on fileno(stdout) */
	if (err < 0) {
	    perror("dup");
	    exit(err);
	}
	close(fdfrom[1]);
	fsync(1);

	fsync(fdto[1]);
	close(fdto[1]);		/* write of pipe fdto not used */
	fsync(fdfrom[0]);
	close(fdfrom[0]);	/* read  of pipe fdfrom not used */

#if USEZLIB
	memset(buffer, 0, (size_t)(orig_len + 3));
	bytestream = gzdopen(0, "r");

	size = 0;
	size = gzread(bytestream, buffer, orig_len);
	gzclose(bytestream);
	fsync(0);
	close(0);          /* say mother it's finished */
	if (size != orig_len) {
	    perror("buffer error");
	}

	err = write(1, buffer, orig_len);
	fsync(1);
	close(1);          /* say mother it's finished */
	if (err < 0) {
	    perror("write");
	    exit(err);
	}
	exit(0);
#else
	setenv("PATH", "/bin:/usr/bin:/usr/X11R6/bin", 1);
	execlp("gunzip", "gunzip", "-fqc", (char*)0);
	perror("execlp");
        exit(-1); /* not reached */
#endif
	} break;
    default:  { /* it's own group */
#ifdef USEFDOPEN
	FILE *charstream;
#endif
	fsync(fdto[0]);
	close(fdto[0]);    /* read  of pipe fdto not used */
	fsync(fdfrom[1]);
	close(fdfrom[1]);  /* write of pipe fdfrom not used */

	err = write(fdto[1], inbuf, ilen);
	fsync(fdto[1]);
	close(fdto[1]);    /* say child it's finished */
	if (err < 0) {
	    perror("write");
	    return err;
	}

	size = 0;
	*output = (char*)malloc((size_t)(orig_len + 3));
	if (!(*output)) {
	    perror("malloc");
	    return err;
	}
	memset(*output, 0, (size_t)(orig_len + 3));
#if USEFDOPEN
	fsync(fdfrom[0]);
	charstream = fdopen(fdfrom[0], "r");
	clearerr(charstream);
	size = fread((void*)(*output), 1, orig_len, charstream);
	fclose(charstream);
	fsync(fdfrom[0]);
	close(fdfrom[0]);    /* say child it's finished */
#else
	while (1) {
	    fsync(fdfrom[0]);
	    mark = read(fdfrom[0], ((*output)+size),
		(orig_len > (PIPE_BUF) ? (PIPE_BUF) : orig_len));
	    if (mark < 0) {
		perror("read");
		return mark;
	    }
	    size += mark;
	    if (mark == 0 || size == orig_len) {
		fsync(fdfrom[0]);
		close(fdfrom[0]);    /* say child it's finished */
		break;
	    }
	    if (size > orig_len) {
		perror("buffer overflow");
		return -1;
	    }
	}
#endif
	} break;
    }
    *olen  = size;
    return 0;
}

#ifdef MAIN
char **extract_pick(char *input, u_int *count)
{
    char *s, *t;
    char **xpm;
    u_int stropen;
    u_int pw = 0, ph = 0, cc = 0, dp = 0;

    xpm = (char**)0;

    if (strncmp(input, "/* XPM */", 9) != 0)
	return xpm;

    s = index(input, '{');
    if (s && s[0] == '{')
	s++;

    *count = 0;

    stropen = 0;
    while (1) {
	t = strtok(s, "\"");
	if (s)
	    s = (char*)0;
	if (!t)
	    break;
	stropen++;
        if (stropen % 2 == 0) {
 	    (*count)++;
	    if ( *count == 1 ) {
		sscanf(t, "%d %d %d %d", &pw, &ph, &cc, &dp);
		xpm = (char**)malloc((size_t)(sizeof(char*) * (ph + cc + 3)));
		xpm[0] = strdup(t);
		xpm[ph + cc + 2] = (char*)0;
	    } else if (xpm) {
		xpm[*count - 1] = strdup(t);
	    } else if ( *count > ph + cc + 3) {
		return xpm;
	    } else
		break;
	}
    }

    return xpm;
}

main()
{
    u_int out_len, len, count;
    char *output;
    char **xpm;

    get_pick(SuSE_Linux_data, SuSE_Linux_len, &output, &out_len);
    xpm = extract_pick(output, &count);

    if (!xpm)
	exit(0);
    for (len = 0; len < count; len++)
	fprintf(stderr, "%s\n", xpm[len]);
}
#endif
